import json
import typing

from starlette import status
from starlette.requests import Request
from starlette.responses import HTMLResponse, JSONResponse, PlainTextResponse, Response

from strawberry.file_uploads.data import replace_placeholders_with_files

from .utils import get_graphiql_html


async def get_http_response(
    request: Request,
    execute: typing.Callable,
    process_result: typing.Callable,
    graphiql: bool,
    root_value: typing.Optional[typing.Any],
    context: typing.Optional[typing.Any],
) -> Response:
    if request.method == "GET":
        if not graphiql:
            return HTMLResponse(status_code=status.HTTP_404_NOT_FOUND)

        html = get_graphiql_html()
        return HTMLResponse(html)

    if request.method == "POST":
        content_type = request.headers.get("Content-Type", "")
        if "application/json" in content_type:
            data = await request.json()
        elif content_type.startswith("multipart/form-data"):
            multipart_data = await request.form()
            operations = json.loads(multipart_data.get("operations", "{}"))
            files_map = json.loads(multipart_data.get("map", "{}"))

            data = replace_placeholders_with_files(
                operations, files_map, multipart_data
            )

        else:
            return PlainTextResponse(
                "Unsupported Media Type",
                status_code=status.HTTP_415_UNSUPPORTED_MEDIA_TYPE,
            )
    else:
        return PlainTextResponse(
            "Method Not Allowed",
            status_code=status.HTTP_405_METHOD_NOT_ALLOWED,
        )

    try:
        query = data["query"]
        variables = data.get("variables")
        operation_name = data.get("operationName")
    except KeyError:
        return PlainTextResponse(
            "No GraphQL query found in the request",
            status_code=status.HTTP_400_BAD_REQUEST,
        )

    result = await execute(
        query,
        variables=variables,
        context=context,
        operation_name=operation_name,
        root_value=root_value,
    )

    response_data = await process_result(request=request, result=result)

    return JSONResponse(response_data, status_code=status.HTTP_200_OK)



class BasePermission:
    """
    Base class for creating permissions
    """

    def has_permission(self, source, info, **kwargs):
        raise NotImplementedError(
            "Permission classes should override has_permission method"
        )



import datetime
import json
import typing

from pygments import highlight, lexers
from pygments.formatters import Terminal256Formatter

from .graphql_lexer import GraphQLLexer


def pretty_print_graphql_operation(
    operation_name: str, query: str, variables: typing.Dict["str", typing.Any]
):  # pragma: no cover
    """Pretty print a GraphQL operation using pygments.

    Won't print introspection operation to prevent noise in the output."""

    if operation_name == "IntrospectionQuery":
        return

    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    print(f"[{now}]: {operation_name or 'No operation name'}")
    print(highlight(query, GraphQLLexer(), Terminal256Formatter()))

    if variables:
        variables_json = json.dumps(variables, indent=4)

        print(highlight(variables_json, lexers.JsonLexer(), Terminal256Formatter()))



from contextlib import contextmanager
from typing import Any, Dict, List

from graphql import MiddlewareManager

from strawberry.types import ExecutionContext

from . import Extension


class ExtensionsRunner:
    extensions: List[Extension]

    def __init__(
        self,
        execution_context: ExecutionContext,
        extensions: List[Extension] = None,
    ):
        self.execution_context = execution_context
        self.extensions = extensions or []

    def _run_on_all_extensions(self, method_name: str, *args, **kwargs):
        for extension in self.extensions:
            getattr(extension, method_name)(*args, **kwargs)

    @contextmanager
    def request(self):
        self._run_on_all_extensions(
            "on_request_start", execution_context=self.execution_context
        )

        yield

        self._run_on_all_extensions(
            "on_request_end", execution_context=self.execution_context
        )

    @contextmanager
    def validation(self):
        self._run_on_all_extensions("on_validation_start")

        yield

        self._run_on_all_extensions("on_validation_end")

    @contextmanager
    def parsing(self):
        self._run_on_all_extensions("on_parsing_start")

        yield

        self._run_on_all_extensions("on_parsing_end")

    def get_extensions_results(self) -> Dict[str, Any]:
        data = {}

        for extension in self.extensions:
            data.update(extension.get_results())

        return data

    def as_middleware_manager(self, *additional_middlewares) -> MiddlewareManager:
        middlewares = tuple(self.extensions) + additional_middlewares

        return MiddlewareManager(*middlewares)
